///////////////////////////////////////////////////////////////////////////////
// FlameProjectile
// By: Dopamine, Kamek
// For: Eternal Damnation
//
// Flamethrower projectile for Eternal Damnation.
//
// 7/30/07 (Kamek)
//		Set TossZ to 0, projectile should shoot straight now
//
///////////////////////////////////////////////////////////////////////////////
class FlameProjectile extends P2Projectile;

///////////////////////////////////////////////////////////////////////////////
// Vars, consts, etc.
///////////////////////////////////////////////////////////////////////////////

var() class<P2Emitter> FireEffectsClass; // Class name of our fire effects
var() class<P2Emitter> FireExplosionClass; // Class name of fire explosion
var() float FireEffectSizeMod; // How quickly the fire effects grow. For "show" only -- does not affect actual collision

var P2Emitter ArrowFire;

var() byte BounceCount;
var() byte BounceMax;
var() bool bDoBounce;

///////////////////////////////////////////////////////////////////////////////
// Set up our fire and attach it to the invisible projectile.
///////////////////////////////////////////////////////////////////////////////
simulated function PostBeginPlay()
{
	Super.PostBeginPlay();

	//Dopamine -- Set the flying arrow on fire
	if (Level.NetMode != NM_DedicatedServer)
	{
  		if(Level.Game == None
  			|| !Level.Game.bIsSinglePlayer)
  			ArrowFire = spawn(FireEffectsClass, self,, Location);
  		else
  			// Send player as owner so it will keep up in slomo time
  			ArrowFire = spawn(FireEffectsClass, Instigator,, Location);
	}

	//log("my ArrowFire is" @ ArrowFire);
	if (ArrowFire != None)
		ArrowFire.SetBase(Self);

	Velocity = GetThrownVelocity(Instigator, Rotation, 0.4);

	// Bounce off of walls and travel faster in enhanced mode.
	if (P2GameInfoSingle(Level.Game).VerifySeqTime() && Instigator.Controller.bIsPlayer)
	{
		Velocity = 3*Velocity;
		bDoBounce = True;
	}
}

///////////////////////////////////////////////////////////////////////////////
// Remove our fire, if it's not already destroyed.
///////////////////////////////////////////////////////////////////////////////
simulated function Destroyed()
{
	//DOPAMINE - Destroy the arrow flame on impact
	if(ArrowFire != None)
	{
		//Dopamine - Destroy the arrow flame so it doesn't just stay forever
		ArrowFire.Destroy();
		ArrowFire = None;
	}

	Super.Destroyed();
}

///////////////////////////////////////////////////////////////////////////////
// Ignore damage
///////////////////////////////////////////////////////////////////////////////
function TakeDamage( int Damage, Pawn instigatedBy, Vector hitlocation,
						Vector momentum, class<DamageType> damageType);

///////////////////////////////////////////////////////////////////////////////
// Dies when it hits a wall/floor
// If we're in enhanced mode, bounce around and set fire to anything we touch
///////////////////////////////////////////////////////////////////////////////
simulated function HitWall( vector HitNormal, actor Wall )
{
	// Generate fire starter ring
	Explode(Location, HitNormal);

	if (bDoBounce)
	{
		BounceCount++;
		if (BounceCount > BounceMax)
			Destroy();
		else
			Velocity = Velocity - 2 * HitNormal * (Velocity Dot HitNormal);
	}
}

///////////////////////////////////////////////////////////////////////////////
// Damage target and die
///////////////////////////////////////////////////////////////////////////////
simulated function ProcessTouch(Actor Other, Vector HitLocation)
{
	if ( !RelatedToMe(Pawn(Other)) )
	{
		// Breaks windows
		if(Window(Other) != None)
			Other.Bump(self);
		else
		{
			// Balance damage differently for MP vs SP
			if(Level.NetMode == NM_Standalone)
				Other.TakeDamage(Damage, instigator, Location, MomentumTransfer * Normal(Velocity), MyDamageType);
			else
				Other.TakeDamage(DamageMP, instigator, Location, MomentumTransfer * Normal(Velocity), MyDamageType);
		}

		// Die without exploding.
		Destroy();
	}
}

///////////////////////////////////////////////////////////////////////////////
// Blow up and generate effects
///////////////////////////////////////////////////////////////////////////////
simulated function Explode(vector HitLocation, vector HitNormal)
{
	//Dopamine - Generate explosion on impact
    spawn(FireExplosionClass, Instigator,, Location);

	if (!bDoBounce)
	    Destroy();
}

///////////////////////////////////////////////////////////////////////////////
// Die if our fire goes out
// Gradually make the flames bigger
///////////////////////////////////////////////////////////////////////////////
simulated event Tick(float Delta)
{
	//log("my ArrowFire is" @ ArrowFire);
	if (ArrowFire == None)
	{
		// Cleans up log output
		ArrowFire = None;
		
		// Our emitter died out, so we die too.
		Destroy();
	}
	else
	{
		// Make our flame bigger
		ArrowFire.Emitters[0].SizeScale[1].RelativeSize += (FireEffectSizeMod * Delta);
	}
}

///////////////////////////////////////////////////////////////////////////////
// Default Properties
///////////////////////////////////////////////////////////////////////////////
defaultproperties
{
	FireEffectsClass=class'FlameProjectileFire'
	FireExplosionClass=class'DynamicFireStarterRingSmall'
	FireEffectSizeMod=2.00
	MyDamageType=class'FireExplodedDamage'
	Speed=500.000000 // edit as desired
    MaxSpeed=2000.000000 // edit as desired
    Damage=25.000000 // edit as desired
	DamageMP=50 // edit as desired
	bRotatetoDesired=false
	bFixedRotationDir=true
	DrawType=DT_None
	CollisionHeight=18 // edit as desired
	CollisionRadius=18 // edit as desired
	bUseCylinderCollision=true
	BounceMax=5
	TossZ=0
}
